@@ -84,17 +84,24 @@ bool tlshd_config_init(const gchar *pathname)
 					  "nl", NULL);
 	tmp = g_key_file_get_integer(tlshd_configuration, "debug",
 				     "delay_done", NULL);
-	tlshd_delay_done = tmp > 0 ? tmp : 0;
+	tlshd_delay_done = tmp > 0 ? (unsigned int)tmp : 0;
 
 	keyrings = g_key_file_get_string_list(tlshd_configuration,
 					      "authenticate",
 					      "keyrings", &length, NULL);
 	if (keyrings) {
 		for (i = 0; i < length; i++) {
+			if (!strcmp(keyrings[i], ".nvme"))
+				continue;
 			tlshd_keyring_link_session(keyrings[i]);
 		}
 		g_strfreev(keyrings);
 	}
+	/*
+	 * Always link the default nvme subsystem keyring into the
+	 * session.
+	 */
+	tlshd_keyring_link_session(".nvme");
 
 	return true;
 }
@@ -186,6 +193,7 @@ bool tlshd_config_get_client_truststore(char **bundle)
 		return false;
 	} else if (access(pathname, F_OK)) {
 		tlshd_log_debug("client x509.truststore pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
@@ -222,6 +230,7 @@ bool tlshd_config_get_client_certs(gnutls_pcert_st *certs,
 		return false;
 	} else if (access(pathname, F_OK)) {
 		tlshd_log_debug("client x509.certificate pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
@@ -268,7 +277,8 @@ bool tlshd_config_get_client_privkey(gnutls_privkey_t *privkey)
 		g_error_free(error);
 		return false;
 	} else if (access(pathname, F_OK)) {
-		tlshd_log_debug("client x095.private_key pathname \"%s\" is not accessible", pathname);
+		tlshd_log_debug("client x509.private_key pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
@@ -321,6 +331,7 @@ bool tlshd_config_get_server_truststore(char **bundle)
 		return false;
 	} else if (access(pathname, F_OK)) {
 		tlshd_log_debug("server x509.truststore pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
@@ -357,6 +368,7 @@ bool tlshd_config_get_server_certs(gnutls_pcert_st *certs,
 		return false;
 	} else if (access(pathname, F_OK)) {
 		tlshd_log_debug("server x509.certificate pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
@@ -378,6 +390,7 @@ bool tlshd_config_get_server_certs(gnutls_pcert_st *certs,
 
 	tlshd_log_debug("Retrieved %u x.509 server certificate(s) from %s",
 			*certs_len, pathname);
+	g_free(pathname);
 	return true;
 }
 
@@ -403,6 +416,7 @@ bool tlshd_config_get_server_privkey(gnutls_privkey_t *privkey)
 		return false;
 	} else if (access(pathname, F_OK)) {
 		tlshd_log_debug("server x509.privkey pathname \"%s\" is not accessible", pathname);
+		g_free(pathname);
 		return false;
 	}
 
